package main

import (
	"context"
	"gitea.timerzz.com/kedaya_haitao/coach-spider/pkg/options"
	"gitea.timerzz.com/kedaya_haitao/coach-spider/product"
	"gitea.timerzz.com/kedaya_haitao/coach-spider/server"
	"gitea.timerzz.com/kedaya_haitao/common/pkg/coach-client"
	"gitea.timerzz.com/kedaya_haitao/common/pkg/database"
	"gitea.timerzz.com/kedaya_haitao/common/pkg/proxy"
	"github.com/gofiber/fiber/v3"
	"github.com/gofiber/fiber/v3/middleware/cors"
	"github.com/golang/glog"
	"gorm.io/gorm/logger"
	"log"
	"log/slog"
	"os"
	"os/signal"
	"time"
)

func main() {
	slog.Info(">>> BEGIN INIT<<<")
	ctx, cancel := signal.NotifyContext(context.Background(), os.Kill, os.Interrupt)
	defer cancel()
	cfg, err := options.LoadConfig()
	if err != nil {
		glog.Fatalf("获取配置失败：%v", err)
	}
	db, err := database.InitDatabase(&cfg.DB)
	if err != nil {
		glog.Fatalf("初始化数据库失败：%v", err)
	}
	db.Logger = logger.New(
		log.New(os.Stdout, "\r\n", log.LstdFlags),
		logger.Config{
			SlowThreshold:             time.Second,
			LogLevel:                  logger.Warn,
			IgnoreRecordNotFoundError: false,
			Colorful:                  true,
		},
	)

	// 代理池
	pool := proxy.NewProxyPool(cfg.Proxy.Subscribes)

	go pool.CronUpdate(ctx, cfg.Proxy.Interval) //定时更新代理

	// coach client
	client := coach_client.USClient(pool)

	productCtl := product.NewController(client, db)

	go productCtl.Run(ctx)

	r := fiber.New()
	r.Use(cors.New())

	r.Get("/health", func(ctx fiber.Ctx) error {
		return ctx.SendString("ok")
	})

	api := r.Group("/api/v1")

	server.NewSpiderSvc(productCtl).RegistryRouter(api)
	//
	//server.NewWatcherController(watcherCtl).RegistryRouter(api)
	//server.NewPusherSvcController(pusherCtl).RegistryRouter(api)
	//server.NewProxySvc(pool).RegistryRouter(api)

	if err = r.Listen(":8080"); err != nil {
		glog.Warningf("server over: %v", err)
	}
}
